-- Prozedur/Funktion proc_del_cdb_them_surf_tex erstellen.
create or replace procedure proc_del_cdb_them_surf_tex 
	(in pcitydb_schema_prefix varchar, in pthematic_surface_id numeric, in plod numeric, in ptheme varchar) 
language plpgsql	
as $$
declare
	from_clause varchar(500);
	query_str varchar(500);
	surface_data_id numeric;
	surface_geometry_id numeric;
	tex_image_id numeric;
	row_count numeric;
	appearance_id numeric;
begin -- main
	from_clause := ' from ' ||
		pcitydb_schema_prefix || 'surface_data sd, ' ||
		pcitydb_schema_prefix || 'textureparam tp, ' ||
		pcitydb_schema_prefix || 'surface_geometry sg where tp.surface_data_id=sd.id and tp.surface_geometry_id=sg.id and ' ||
		'sd.id in (select surface_data_id from ' ||
		pcitydb_schema_prefix || 'appear_to_surface_data, ' ||
		pcitydb_schema_prefix || 'appearance where appear_to_surface_data.appearance_id=appearance.id and appearance.theme=''' ||
		ptheme || ''') and sg.root_id in (select lod' || 
		plod || '_multi_surface_id from ' || 
		pcitydb_schema_prefix || 'thematic_surface where id=' || pthematic_surface_id || ')';
		
	-- loop 1 (SURFACE_DATA_ID and SURFACE_GEOMETRY_ID)		
	for surface_data_id, surface_geometry_id in 
	execute format('select sd.id as surface_data_id, sg.id as surface_geometry_id %s', from_clause)
	loop
		-- 1) delete from TEXTUREPARAM
		query_str := 'delete from ' || pcitydb_schema_prefix || 
			'textureparam where surface_geometry_id=' || surface_geometry_id || ' and surface_data_id=' || surface_data_id;
		--raise notice 'execute %', query_str;
		execute query_str;
	end loop;
	
	-- loop 2 (distinct SURFACE_DATA_ID)
	for surface_data_id in 
	execute format('select distinct sd.id as surface_data_id %s', from_clause)
	loop
		-- get TEX_IMAGE_ID
		query_str := 'select tex_image_id from ' || pcitydb_schema_prefix || 'surface_data where id=' || surface_data_id;
		--raise notice 'execute %', query_str;
		execute query_str into tex_image_id;
		
		if tex_image_id >= 0 then
			-- reset tex_image_id in surface_data
			query_str := 'update ' || pcitydb_schema_prefix || 'surface_data set tex_image_id=null where id=' || surface_data_id;
			--raise notice 'execute %', query_str;
			execute query_str;

			-- delete from table tex_image
			query_str := 'delete from ' || pcitydb_schema_prefix || 'tex_image where id=' || tex_image_id;
			--raise notice 'execute %', query_str;
			execute query_str;
		end if;
		
		-- if no other texture param row refers to my surface_data_id, go on with deleting
		query_str := 'select count(surface_geometry_id) from ' || pcitydb_schema_prefix || 'textureparam where surface_data_id=' || surface_data_id;
		execute query_str into row_count;
		--raise notice 'remaining textureparam row count: %', row_count;
		if row_count=0 then
			-- 2) delete from appear_to_surface_data
			query_str := 'delete from ' || pcitydb_schema_prefix || 'appear_to_surface_data where surface_data_id=' || surface_data_id;
			--raise notice 'execute %', query_str;
			execute query_str;
			-- 3) delete from surface_data
			query_str := 'delete from ' || pcitydb_schema_prefix || 'surface_data where id=' || surface_data_id;
			--raise notice 'execute %', query_str;
			execute query_str;
		end if;
	end loop;

	-- loop 3 (distinct APPEARANCE_ID)
	for appearance_id in 
	execute 'select distinct appearance_id from ' ||
			pcitydb_schema_prefix || 'appear_to_surface_data where surface_data_id in (select distinct sd.id from ' ||
			pcitydb_schema_prefix || 'surface_data sd, ' ||
			pcitydb_schema_prefix || 'textureparam tp, ' ||
			pcitydb_schema_prefix || 'surface_geometry sg where tp.surface_data_id=sd.id and tp.surface_geometry_id=sg.id and ' ||
			'sg.root_id in (select lod' || 
			plod || '_multi_surface_id from ' || 
			pcitydb_schema_prefix || 'thematic_surface where id=' || pthematic_surface_id || ')) and appearance_id in (select id from ' ||
			pcitydb_schema_prefix || 'appearance where theme=''' ||
			ptheme || ''')'
	loop
		-- if no other appear_to_surface_data row refers to my appearance_id, delete appearance
		query_str := 'select count(surface_data_id) from ' || pcitydb_schema_prefix || 'appear_to_surface_data where appearance_id=' || appearance_id;
		execute query_str into row_count using appearance_id;
		raise notice 'remaining textureparam row count: %', row_count;
		if row_count=0 then
			-- 4) delete from appearance
			query_str := 'delete from ' || pcitydb_schema_prefix || 'appearance where id=' || appearance_id;
			raise notice 'execute %', query_str;
			execute query_str;
		end if;
	end loop;
end;$$;

-- Prozedur/Funktion PROC_DEL_CDB_BUILDING_TEXTURE erstellen.
create or replace procedure proc_del_cdb_building_texture 
	(in pcitydb_schema_prefix varchar, in pbuilding_id numeric, in plod numeric, in ptheme varchar) 
language plpgsql	
as $$
declare
	query_str varchar(500);
	thematic_surface_id numeric;
begin
	-- query for all thematic surfaces
	query_str := 'select id from ' ||
		pcitydb_schema_prefix || 'thematic_surface where building_id in (select id from ' ||
		pcitydb_schema_prefix || 'building where building_root_id=' || pbuilding_id || ')';
	raise notice 'query_str %', query_str;
	
	-- loop over all thematic surfaces
	for thematic_surface_id in
	execute query_str
	loop
		call proc_del_cdb_them_surf_tex(pcitydb_schema_prefix, thematic_surface_id, plod, ptheme);
	end loop;
end;$$;

-- Prozedur/Funktion PROC_DEL_CDB_MODEL_TEXTURE erstellen.
create or replace procedure proc_del_cdb_model_texture 
	(in pcitydb_schema_prefix varchar, in pmodel_name varchar, in plod numeric, in ptheme varchar) 
language plpgsql	
as $$
declare
	building_id numeric;
begin
	for building_id in
	execute format('select building_id from modelbuilding where model in (select model from model where namen=%s)', quote_literal(pmodel_name))
	loop
		--raise notice 'call proc_del_cdb_building_texture(%,%,%,%)', quote_literal(pcitydb_schema_prefix), building_id, plod, quote_literal(ptheme);
		call proc_del_cdb_building_texture(pcitydb_schema_prefix, building_id, plod, ptheme);
	end loop;
end;$$;

