-- 3 Possibilities for Addresses:
-- a) 2 views or tables in an external database;scheme with the following columns:
--    a1) STREET_NAME, STREET_ID
--    a2) ADR_X, ADR_Y, STREET_NUMBER, OBJECT_ID, OBJ_X, OBJ_Y, STREET_NAME, STREET_ID
--    and the synonyms GIS_V_STREETS and GIS_V_OBJECTADRESS refering to them.
-- b) 2 views in the CITYMAN-scheme with the columns as above viewing to external tables
--    and the synonyms GIS_V_STREETS and GIS_V_OBJECTADRESS refering to them.
-- c) 2 tables in the CITYMAN-scheme with the columns as above
--    and the synonyms GIS_V_STREETS and GIS_V_OBJECTADRESS refering to them.
-- Variante (c) is installed per default (with table names STREET_TEMPLATE and ADDRESS_TEMPLATE)
-- and can be changed interactively afterwards.

-- The Following two views are only as template for variante (b)
create or replace view "v_address" ("adr_x",
		"adr_y","street_number","object_id","obj_x","obj_y",
		"street_name","street_id") as 
	select address_template.adr_x, address_template.adr_y, 
	address_template.street_number, address_template.object_id, 
	address_template.obj_x, address_template.obj_y, 
	street_template.name as street_name,
	street_template.street_key as street_id 
	from address_template, street_template 
	where ( address_template.street_key = street_template.street_key )
	
;

create or replace view "gis_v_objectadress" ("adr_x",
		"adr_y","street_number","object_id","obj_x","obj_y",
		"street_name","street_id") as 
	select address_template.adr_x, address_template.adr_y, 
	address_template.street_number, address_template.object_id, 
	address_template.obj_x, address_template.obj_y, 
	street_template.name as street_name,
	street_template.street_key as street_id 
	from address_template, street_template 
	where ( address_template.street_key = street_template.street_key )
	
;

create or replace view "v_street" ("street_name", "street_id") as 
	select name as street_name, street_key as street_id 
	from street_template 
;

create or replace view "gis_v_street" ("street_name", "street_id") as 
	select name as street_name, street_key as street_id 
	from street_template
;

-- the following views are obligatory! they use the synonym gis_v_objectadress

create or replace view v_gis_modelobjects as 
	select modelunit.model, unit.unit, unit_number, unitid, unit.namen, gis_v_objectadress.street_name, gis_v_objectadress.street_number, gis_v_objectadress.object_id, unitgis.street_number_int 
	from modelunit, unit, unitgis, gis_v_objectadress 
	where ( ( unit.unit = modelunit.unit ) and ( unitgis.unit = unit.unit ) and ( gis_v_objectadress.object_id = unitgis.giskey ) );
;

create or replace view "v_gis_objects" as	
	select unit.unit, unit_number, unitid, gis_v_objectadress.street_name, unit.namen
	from unit, unitgis, gis_v_objectadress
	where ( unitgis.unit = unit.unit and gis_v_objectadress.object_id = unitgis.giskey );
;

-- views für die letzte gültige;stabile versionsnummer
create or replace view "v_last_valid_versionnumber" as	
	select unitversion.unit, max(unitversion.versionnumber) as versionnumber
	from unitversion
	where unitversion.state <> 3
	group by unitversion.unit;
;
	
create or replace view "v_last_stable_versionnumber" as	
	select unitversion.unit, max(unitversion.versionnumber) as versionnumber
	from unitversion
	where unitversion.state <> 2 and unitversion.state <> 3
	group by unitversion.unit;
;

-- new views in schema 2.6
create or replace view v_last_valid_version as 
	select unitversion.unitversion, unitversion.versionnumber, unitversion.versiondate, unitversion.unit, unitversion.state, unitversion."comment" 
	from unitversion, v_last_valid_versionnumber 
	where unitversion.unit=v_last_valid_versionnumber.unit 
	and unitversion.versionnumber=v_last_valid_versionnumber.versionnumber;
;

create or replace view v_last_stable_version as 
	select unitversion.unitversion, unitversion.versionnumber, unitversion.versiondate, unitversion.unit, unitversion.state, unitversion."comment" 
	from unitversion, v_last_stable_versionnumber 
	where unitversion.unit=v_last_stable_versionnumber.unit 
	and unitversion.versionnumber=v_last_stable_versionnumber.versionnumber;
;

-- new version in schema 2.7   ********
create or replace view v_model_texture_image as 
	select distinct modelunit.model, image.image 
	from modelunit, v_last_stable_version, object, elementcomplex, element, texture, image 
	where 
		v_last_stable_version.unit = modelunit.unit and 
		object.unitversion = v_last_stable_version.unitversion and 
		elementcomplex.object = object.object and 
		element.elementcomplex = elementcomplex.elementcomplex and 
		texture.element = element.element and 
		image.image = texture.image;

create or replace view "v_modelunits" as
	select model.namen as model_namen, unit.namen
    as unit_namen, unit.unit_number, unit.unitid, modelunit.model, modelunit.unit
    from model, unit, modelunit
    where ( ( modelunit.model = model.model ) and ( unit.unit = modelunit.unit ) );
;

create or replace view "v_unitexport" ("data","namen",
    "model","unit","exportdata","lod","type") as 
    select exportdata.data, model.namen, 
    modelunit.model, modelunit.unit, unitexport.
    exportdata, unitexport.lod, unitexport.type
    from exportdata, model, modelunit,
    unitexport
    where ( ( modelunit.model = model.model )
    and ( unitexport.unit =  modelunit.unit)
    and ( exportdata.exportdata =   unitexport.exportdata)  ) 
;

-- new version in schema 2.6
create or replace view v_unit_texture_image as 
	select distinct unit.unit, unitversion.unitversion, image.image 
	from unit, unitversion, object, elementcomplex, element, texture, image 
	where 
		unitversion.unit = unit.unit and 
		object.unitversion = unitversion.unitversion and 
		elementcomplex.object = object.object and 
		element.elementcomplex = elementcomplex.elementcomplex and 
		texture.element = element.element and 
		image.image = texture.image;

-- new views in schema 4.2
create or replace view v_last_geom_versionnumber as 
	select unitversion.unit, max(unitversion.versionnumber) as versionnumber 
	from unitversion 
	where unitversion.state in (1,2) 
	group by unitversion.unit;
;

create or replace view v_last_geom_version as 
	select unitversion.unitversion, unitversion.versionnumber, unitversion.versiondate, unitversion.unit, unitversion.state, unitversion."comment" 
	from unitversion, v_last_geom_versionnumber 
	where unitversion.unit=v_last_geom_versionnumber.unit 
	and unitversion.versionnumber=v_last_geom_versionnumber.versionnumber;
;